@@ -107,18 +107,19 @@ def _convert_to_nice_slice(r, N, name="range"):
     elif len(r) == 2:
         r2 = list(r)
         if r2[0] < 0:
-            r2[0] += N
+            r2[0] = r2[0] + N
         if r2[1] < 0:
-            r2[1] += N
+            r2[1] = r2[1] + N
         if r2[0] > r2[1]:
             raise ValueError("{} start ({}) is larger than end ({})"
                              .format(name, *r2))
         # Lists uses inclusive end, we need exclusive end
         temp_slice = slice(r2[0], r2[1] + 1)
+    elif len(r) == 3:
+        # Convert 3 element list to slice object
+        temp_slice = slice(r[0],r[1],r[2])
     else:
-        raise ValueError("Couldn't convert {} ('{}') to slice. Please pass a "
-                         "slice(start, stop, step) if you need to set a step."
-                         .format(name, r))
+        raise ValueError("Couldn't convert {} ('{}') to slice".format(name, r))
 
     # slice.indices converts None to actual values
     return slice(*temp_slice.indices(N))
@@ -142,8 +143,10 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
         Path to data files (default: ".")
     prefix : str, optional
         File prefix (default: "BOUT.dmp")
-    yguards : bool, optional
+    yguards : bool or "include_upper", optional
         Collect Y boundary guard cells? (default: False)
+        If yguards=="include_upper" the y-boundary cells from the upper (second) target
+        are also included.
     xguards : bool, optional
         Collect X boundary guard cells? (default: True)
         (Set to True to be consistent with the definition of nx)
@@ -191,6 +194,12 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
 
         f = getDataFile(0)
 
+        if varname not in f.keys():
+            if strict:
+                raise ValueError("Variable '{}' not found".format(varname))
+            else:
+                varname = findVar(varname, f.list())
+
         dimensions = f.dimensions(varname)
 
         try:
@@ -210,6 +219,10 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
             nx = f["nx"] - 2*mxg
         if yguards:
             ny = f["ny"] + 2*myg
+            if yguards == "include_upper" and f["jyseps2_1"] != f["jyseps1_2"]:
+                # Simulation has a second (upper) target, with a second set of y-boundary
+                # points
+                ny = ny + 2*myg
         else:
             ny = f["ny"]
         nz = f["MZ"]
@@ -235,9 +248,9 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
         if not yguards:
             yind = slice(yind.start+myg, yind.stop+myg, yind.step)
 
-        if len(dimensions) == ():
+        if dimensions == ():
             ranges = []
-        elif dimensions == ('t'):
+        elif dimensions == ('t',):
             ranges = [tind]
         elif dimensions == ('x', 'y'):
             # Field2D
@@ -258,8 +271,9 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
             # evolving Field3D
             ranges = [tind, xind, yind, zind]
         else:
-            raise ValueError("Variable has incorrect dimensions ({})"
-                             .format(dimensions))
+            # Not a Field, so do not support slicing.
+            # For example, may be a string.
+            ranges = None
 
         data = f.read(varname, ranges)
         var_attributes = f.attributes(varname)
@@ -362,6 +376,18 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
 
     if yguards:
         ny = mysub * nype + 2*myg
+        if yguards == "include_upper" and f["jyseps2_1"] != f["jyseps1_2"]:
+            # Simulation has a second (upper) target, with a second set of y-boundary
+            # points
+            ny = ny + 2*myg
+            ny_inner = f["ny_inner"]
+            yproc_upper_target = ny_inner // mysub - 1
+            if f["ny_inner"] % mysub != 0:
+                raise ValueError("Trying to keep upper boundary cells but "
+                                 "mysub={} does not divide ny_inner={}"
+                                 .format(mysub, ny_inner))
+        else:
+            yproc_upper_target = None
     else:
         ny = mysub * nype
 
@@ -375,11 +401,19 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
     zsize = int(np.ceil(float(zind.stop - zind.start)/zind.step))
     tsize = int(np.ceil(float(tind.stop - tind.start)/tind.step))
 
-    if ndims == 1:
-        if tind is None:
-            data = f.read(varname)
+    if not any(dim in dimensions for dim in ('x', 'y', 'z')):
+        # Not a Field (i.e. no spatial dependence) so only read from the 0'th file
+        if 't' in dimensions:
+            if not dimensions[0] == 't':
+                # 't' should be the first dimension in the list if present
+                raise ValueError(
+                    varname + " has a 't' dimension, but it is not the first dimension "
+                    "in dimensions=" + str(dimensions)
+                )
+            data = f.read(varname, ranges = [tind] + (ndims - 1) * [None])
         else:
-            data = f.read(varname, ranges=[tind])
+            # No time or space dimensions, so no slicing
+            data = f.read(varname)
         if datafile_cache is None:
             # close the DataFile if we are not keeping it in a cache
             f.close()
@@ -427,6 +461,9 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
                     inrange = False
                 if ystart < myg:
                     ystart = myg
+            # and lower y boundary at upper target
+            if yproc_upper_target is not None and pe_yind - 1 == yproc_upper_target:
+                ystart = ystart - myg
 
             # Upper y boundary
             if pe_yind == (nype - 1):
@@ -440,11 +477,18 @@ def collect(varname, xind=None, yind=None, zind=None, tind=None, path=".",
                     inrange = False
                 if ystop > (mysub + myg):
                     ystop = (mysub + myg)
+            # upper y boundary at upper target
+            if yproc_upper_target is not None and pe_yind == yproc_upper_target:
+                ystop = ystop + myg
 
             # Calculate global indices
             ygstart = ystart + pe_yind * mysub
             ygstop = ystop + pe_yind * mysub
 
+            if yproc_upper_target is not None and pe_yind > yproc_upper_target:
+                ygstart = ygstart + 2*myg
+                ygstop = ygstop + 2*myg
+
         else:
             # Get local ranges
             ystart = yind.start - pe_yind*mysub + myg
